"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodejsFunction = void 0;
const lambda = __importStar(require("@aws-cdk/aws-lambda"));
const es = __importStar(require("esbuild"));
const path = __importStar(require("path"));
const ramda_1 = require("ramda");
const pack_externals_1 = require("./pack-externals");
const utils_1 = require("./utils");
const BUILD_FOLDER = '.build';
const DEFAULT_BUILD_OPTIONS = {
    bundle: true,
    target: utils_1.NodeMajorESMap[utils_1.nodeMajorVersion()],
};
/**
 * A Node.js Lambda function bundled using `esbuild`
 */
class NodejsFunction extends lambda.Function {
    constructor(scope, id, props = {}) {
        var _a, _b, _c, _d;
        if (props.runtime && props.runtime.family !== lambda.RuntimeFamily.NODEJS) {
            throw new Error('Only `NODEJS` runtimes are supported.');
        }
        const projectRoot = utils_1.findProjectRoot(props.rootDir);
        if (!projectRoot) {
            throw new Error('Cannot find root directory. Please specify it with `rootDir` option.');
        }
        const withDefaultOptions = ramda_1.mergeRight(DEFAULT_BUILD_OPTIONS);
        const buildOptions = withDefaultOptions((_a = props.esbuildOptions) !== null && _a !== void 0 ? _a : {});
        const exclude = ramda_1.union(props.exclude || [], ['aws-sdk']);
        const packager = (_b = props.packager) !== null && _b !== void 0 ? _b : true;
        const handler = (_c = props.handler) !== null && _c !== void 0 ? _c : 'index.handler';
        const defaultRuntime = utils_1.nodeMajorVersion() >= 12
            ? lambda.Runtime.NODEJS_12_X
            : lambda.Runtime.NODEJS_10_X;
        const runtime = (_d = props.runtime) !== null && _d !== void 0 ? _d : defaultRuntime;
        const entry = utils_1.extractFileName(projectRoot, handler);
        es.buildSync(Object.assign(Object.assign({}, buildOptions), { external: ramda_1.union(exclude, buildOptions.external || []), entryPoints: [path.join(projectRoot, entry)], outdir: path.join(projectRoot, BUILD_FOLDER, path.dirname(entry)), platform: 'node' }));
        if (packager) {
            pack_externals_1.packExternalModules(ramda_1.without(exclude, buildOptions.external || []), projectRoot, path.join(projectRoot, BUILD_FOLDER), packager !== true ? packager : undefined);
        }
        super(scope, id, Object.assign(Object.assign({}, props), { runtime, code: lambda.Code.fromAsset(path.join(projectRoot, BUILD_FOLDER)), handler }));
    }
}
exports.NodejsFunction = NodejsFunction;
